#!/usr/bin/env python

#############################################
#Author:        Damiano Barboni <barboni@meeo.it>
#Version:       0.0.1
#Description:   Test parallel wcs request
#Changelog:     Fri Dec  27 09:54:39 CET 2012
#               First version for resMEEO
#############################################

import httplib
import urllib
import sys
import threading
import random
import unittest
import subprocess
import time
import os

####################################################
######## global variables definition ###############
####################################################

WCS_host       = "earthserverclimate"
WCS_port       = 8080
#WCS_request    = "/petascope/wcs2?service=WCS&Request=GetCoverage&version=2.0.0&subsetX=x(501166,501166)&subsetY=y(4429057,4429057)&subsetT=t(2001-01-01,2001-12-31)&CoverageId=AT2_SPCL56_32632_400"
WCS_request    = "/petascope/wcs2?service=WCS&Request=GetCoverage&version=2.0.0&subsetX=x(679276,679276)&subsetY=y(4929231,4929231)&subsetT=t(2005-01-01,2010-12-31)&CoverageId=CRESCO_PM10_32632_20000"
N_REQUEST      = 10

######################################################
        
class requestThread(threading.Thread):
    def __init__(self, i):
        threading.Thread.__init__(self)
        self.th_number = i
    
    #run method       
    def run(self):
        start = time.time()
        print "Thread %d STARTED" %(self.th_number)
        
        #get xml response
        conn = httplib.HTTPConnection(WCS_host, WCS_port)
        conn.request("GET", WCS_request)
        r = conn.getresponse()
        #check if response status is 200
        r_status = r.status
        if 200 <= r_status <= 299:
            xml = r.read()
        else:
            xml = ("Tread %d - Unable to get data from WCS server. Bad response status N %d returned by WCS server" %(self.th_number, r_status))
                    
        filename = "wcs_output_%d.xml" %(self.th_number)
            
        #write output to file
        out_file = open(filename,"w")
        out_file.write(xml)
        out_file.close()
            
        print "Thread %d ENDED in %f seconds" %(self.th_number,time.time()-start)
        sys.exit(0)

class TestParallelRequest(unittest.TestCase):
    
    #initialize test suite
    def setUp(self):
        self.test_passed = False

    def wcs_parallel(self):
        #thread generation
        thread_list = []
        for i in range(0, N_REQUEST):
            thread_list.append(requestThread(i))
            #thread execution
            thread_list[i].start()
        
        #Wait for the thread_list tasks to finish
        for i in range(0,len(thread_list)):
            thread_list[i].join()
        
        #check MD5
        first_MD5 = None
        for  i in range(0, N_REQUEST):
            filename = "wcs_output_%d.xml" %(i)
            if os.path.isfile(filename):
                command = "md5sum %s" %(filename)
                p = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                retval = p.wait()
                self.assertEqual(retval,0,p.stderr.read())
                md5 = p.stdout.read().split(" ")[0]
                if first_MD5 == None:
                    #for the first file only save the MD5, other files must have the same code
                    first_MD5 = md5
                else:
                    self.assertEqual(md5, first_MD5, "Test Failed - md5 for file %s is different than previous" %(filename))
        
        #search for Exceptions on wcs xml resposne
        for  i in range(0, N_REQUEST):
            filename = "wcs_output_%d.xml" %(i)
            self.assertFalse('Exception' in open(filename).read(), "Test Failed - file %s contains an exception:\n%s" %(filename,open(filename).read()))
        
        #test is passed
        self.test_passed = True
        
            
        
        
        
    def tearDown(self):
        if not self.test_passed:
            print "Unittest system won't remove output file automatically"
        
        else:
            print "Unittest system will remove output file automatically"
            for  i in range(0, N_REQUEST):
                filename = "wcs_output_%d.xml" %(i)
                if os.path.isfile(filename):
                    os.remove(filename)
            

        

def makeSuite():
    suite = unittest.TestSuite()
    suite.addTest(TestParallelRequest('wcs_parallel'))

    return suite


if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=3).run(makeSuite())    
    
